DROP TABLE spops_user;

DROP TABLE spops_group;

DROP TABLE spops_group_user;

DROP TABLE spops_security;

DROP TABLE spops_doodad;

DROP SEQUENCE sp_user_seq;

DROP SEQUENCE sp_group_seq;

DROP SEQUENCE sp_security_seq;

DROP SEQUENCE sp_doodad_seq;
